/*
 *                            Yeppp! library header
 *                   This file is auto-generated by Peach-Py,
 *        Portable Efficient Assembly Code-generator in Higher-level Python,
 *                    part of the Yeppp! library infrastrure
 * 
 * This file is part of Yeppp! library and licensed under the New BSD license.
 * 
 * Copyright (C) 2010-2012 Marat Dukhan
 * Copyright (C) 2012-2013 Georgia Institute of Technology
 * All rights reserved.
 *  
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Georgia Institute of Technology nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <yepPredefines.h>
#include <yepTypes.h>

#ifdef __cplusplus
	extern "C" {
#endif

/** @defgroup yepCore yepCore.h: basic arithmetic operations. */

/** @name	Addition */
/**@{*/
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 8-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 8-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 8-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 8-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V8uV8u_V8u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep8u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 8-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 8-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 16-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V8uV8u_V16u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two signed 8-bit integer arrays, producing an array of signed 16-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 8-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of signed 8-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of signed 16-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V8sV8s_V16s(const Yep8s *YEP_RESTRICT x, const Yep8s *YEP_RESTRICT y, Yep16s *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 16-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 16-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 16-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V16uV16u_V16u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 16-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 16-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 32-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V16uV16u_V32u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two signed 16-bit integer arrays, producing an array of signed 32-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 16-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of signed 16-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of signed 32-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V16sV16s_V32s(const Yep16s *YEP_RESTRICT x, const Yep16s *YEP_RESTRICT y, Yep32s *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 32-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 32-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 32-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32uV32u_V32u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 32-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 32-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 64-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32uV32u_V64u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two signed 32-bit integer arrays, producing an array of signed 64-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 32-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of signed 32-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of signed 64-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32sV32s_V64s(const Yep32s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, Yep64s *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two unsigned 64-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 64-bit integer elements to be added.
 * @param[in]	y	Pointer the second array of unsigned 64-bit integer elements to be added.
 * @param[out]	sum	Pointer the array of unsigned 64-bit integer elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V64uV64u_V64u(const Yep64u *YEP_RESTRICT x, const Yep64u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two single precision (32-bit) floating-point arrays, producing an array of single precision (32-bit) floating-point elements.
 * @param[in]	x	Pointer the first array of single precision (32-bit) floating-point elements to be added.
 * @param[in]	y	Pointer the second array of single precision (32-bit) floating-point elements to be added.
 * @param[out]	sum	Pointer the array of single precision (32-bit) floating-point elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V32fV32f_V32f(const Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, Yep32f *YEP_RESTRICT sum, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Adds corresponding elements in two double precision (64-bit) floating-point arrays, producing an array of double precision (64-bit) floating-point elements.
 * @param[in]	x	Pointer the first array of double precision (64-bit) floating-point elements to be added.
 * @param[in]	y	Pointer the second array of double precision (64-bit) floating-point elements to be added.
 * @param[out]	sum	Pointer the array of double precision (64-bit) floating-point elements where the pairwise sums will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a sum argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a sum argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Add_V64fV64f_V64f(const Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, Yep64f *YEP_RESTRICT sum, YepSize length);
/**@}*/

/** @name	Subtraction */
/**@{*/
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 8-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 8-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 8-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 8-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V8uV8u_V8u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep8u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 8-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 8-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 16-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V8uV8u_V16u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two signed 8-bit integer arrays, producing an array of signed 16-bit integer elements.
 * @param[in]	x	Pointer the minuend array of signed 8-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of signed 8-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of signed 16-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V8sV8s_V16s(const Yep8s *YEP_RESTRICT x, const Yep8s *YEP_RESTRICT y, Yep16s *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 16-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 16-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 16-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V16uV16u_V16u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 16-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 16-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 32-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V16uV16u_V32u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two signed 16-bit integer arrays, producing an array of signed 32-bit integer elements.
 * @param[in]	x	Pointer the minuend array of signed 16-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of signed 16-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of signed 32-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V16sV16s_V32s(const Yep16s *YEP_RESTRICT x, const Yep16s *YEP_RESTRICT y, Yep32s *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 32-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 32-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 32-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32uV32u_V32u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 32-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 32-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 64-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32uV32u_V64u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two signed 32-bit integer arrays, producing an array of signed 64-bit integer elements.
 * @param[in]	x	Pointer the minuend array of signed 32-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of signed 32-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of signed 64-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32sV32s_V64s(const Yep32s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, Yep64s *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two unsigned 64-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the minuend array of unsigned 64-bit integer elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of unsigned 64-bit integer elements to be subtracted.
 * @param[out]	difference	Pointer the array of unsigned 64-bit integer elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V64uV64u_V64u(const Yep64u *YEP_RESTRICT x, const Yep64u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two single precision (32-bit) floating-point arrays, producing an array of single precision (32-bit) floating-point elements.
 * @param[in]	x	Pointer the minuend array of single precision (32-bit) floating-point elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of single precision (32-bit) floating-point elements to be subtracted.
 * @param[out]	difference	Pointer the array of single precision (32-bit) floating-point elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V32fV32f_V32f(const Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, Yep32f *YEP_RESTRICT difference, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Subtracts corresponding elements in two double precision (64-bit) floating-point arrays, producing an array of double precision (64-bit) floating-point elements.
 * @param[in]	x	Pointer the minuend array of double precision (64-bit) floating-point elements to be subtracted from.
 * @param[in]	y	Pointer the subtrahend array of double precision (64-bit) floating-point elements to be subtracted.
 * @param[out]	difference	Pointer the array of double precision (64-bit) floating-point elements where the pairwise differences will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a sum.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a difference argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a difference argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Subtract_V64fV64f_V64f(const Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, Yep64f *YEP_RESTRICT difference, YepSize length);
/**@}*/

/** @name	Multiplication */
/**@{*/
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 8-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 8-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 8-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 8-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8uV8u_V8u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep8u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two unsigned 8-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 8-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 8-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 16-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8uV8u_V16u(const Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two signed 8-bit integer arrays, producing an array of signed 16-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 8-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of signed 8-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of signed 16-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8sV8s_V16s(const Yep8s *YEP_RESTRICT x, const Yep8s *YEP_RESTRICT y, Yep16s *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 16-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 16-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 16-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 16-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16uV16u_V16u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep16u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two unsigned 16-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 16-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 16-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 32-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16uV16u_V32u(const Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two signed 16-bit integer arrays, producing an array of signed 32-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 16-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of signed 16-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of signed 32-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16sV16s_V32s(const Yep16s *YEP_RESTRICT x, const Yep16s *YEP_RESTRICT y, Yep32s *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 32-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 32-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 32-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 32-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32uV32u_V32u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep32u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two unsigned 32-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 32-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 32-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 64-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32uV32u_V64u(const Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two signed 32-bit integer arrays, producing an array of signed 64-bit integer elements.
 * @param[in]	x	Pointer the first array of signed 32-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of signed 32-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of signed 64-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE4.1</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32sV32s_V64s(const Yep32s *YEP_RESTRICT x, const Yep32s *YEP_RESTRICT y, Yep64s *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two unsigned 64-bit integer arrays, producing an array of unsigned 64-bit integer elements.
 * @param[in]	x	Pointer the first array of unsigned 64-bit integer elements to be multiplied.
 * @param[in]	y	Pointer the second array of unsigned 64-bit integer elements to be multiplied.
 * @param[out]	product	Pointer the array of unsigned 64-bit integer elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V64uV64u_V64u(const Yep64u *YEP_RESTRICT x, const Yep64u *YEP_RESTRICT y, Yep64u *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two single precision (32-bit) floating-point arrays, producing an array of single precision (32-bit) floating-point elements.
 * @param[in]	x	Pointer the first array of single precision (32-bit) floating-point elements to be multiplied.
 * @param[in]	y	Pointer the second array of single precision (32-bit) floating-point elements to be multiplied.
 * @param[out]	product	Pointer the array of single precision (32-bit) floating-point elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32fV32f_V32f(const Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, Yep32f *YEP_RESTRICT product, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Multiples corresponding elements in two double precision (64-bit) floating-point arrays, producing an array of double precision (64-bit) floating-point elements.
 * @param[in]	x	Pointer the first array of double precision (64-bit) floating-point elements to be multiplied.
 * @param[in]	y	Pointer the second array of double precision (64-bit) floating-point elements to be multiplied.
 * @param[out]	product	Pointer the array of double precision (64-bit) floating-point elements where the pairwise products will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y, and @a product.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a product argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a product argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V64fV64f_V64f(const Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, Yep64f *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8uS8u_V8u(const Yep8u *YEP_RESTRICT x, Yep8u y, Yep8u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8uS8u_V16u(const Yep8u *YEP_RESTRICT x, Yep8u y, Yep16u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V8sS8s_V16s(const Yep8s *YEP_RESTRICT x, Yep8s y, Yep16s *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16uS16u_V16u(const Yep16u *YEP_RESTRICT x, Yep16u y, Yep16u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16uS16u_V32u(const Yep16u *YEP_RESTRICT x, Yep16u y, Yep32u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V16sS16s_V32s(const Yep16s *YEP_RESTRICT x, Yep16s y, Yep32s *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32uS32u_V32u(const Yep32u *YEP_RESTRICT x, Yep32u y, Yep32u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32uS32u_V64u(const Yep32u *YEP_RESTRICT x, Yep32u y, Yep64u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32sS32s_V64s(const Yep32s *YEP_RESTRICT x, Yep32s y, Yep64s *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V64uS64u_V64u(const Yep64u *YEP_RESTRICT x, Yep64u y, Yep64u *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V32fS32f_V32f(const Yep32f *YEP_RESTRICT x, Yep32f y, Yep32f *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_V64fS64f_V64f(const Yep64f *YEP_RESTRICT x, Yep64f y, Yep64f *YEP_RESTRICT product, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV8uV8u_IV8u(Yep8u *YEP_RESTRICT x, const Yep8u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV16uV16u_IV16u(Yep16u *YEP_RESTRICT x, const Yep16u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV32uV32u_IV32u(Yep32u *YEP_RESTRICT x, const Yep32u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV64uV64u_IV64u(Yep64u *YEP_RESTRICT x, const Yep64u *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV32fV32f_IV32f(Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV64fV64f_IV64f(Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV8uS8u_IV8u(Yep8u *YEP_RESTRICT x, Yep8u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV16uS16u_IV16u(Yep16u *YEP_RESTRICT x, Yep16u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV32uS32u_IV32u(Yep32u *YEP_RESTRICT x, Yep32u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV64uS64u_IV64u(Yep64u *YEP_RESTRICT x, Yep64u y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV32fS32f_IV32f(Yep32f *YEP_RESTRICT x, Yep32f y, YepSize length);
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_Multiply_IV64fS64f_IV64f(Yep64f *YEP_RESTRICT x, Yep64f y, YepSize length);
/**@}*/

/** @name	Dot product */
/**@{*/
/**
 * @ingroup	yepCore
 * @brief	Computes the dot product of single precision (32-bit) floating-point elements in two arrays.
 * @param[in]	x	Pointer the first vector of elements.
 * @param[in]	y	Pointer the second vector of elements.
 * @param[out]	dotProduct	Pointer the variable where the dot product values will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a dotProduct argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a dotProduct argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE3</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, FMA3</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX, FMA4</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_DotProduct_V32fV32f_S32f(const Yep32f *YEP_RESTRICT x, const Yep32f *YEP_RESTRICT y, Yep32f *YEP_RESTRICT dotProduct, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes the dot product of double precision (64-bit) floating-point elements in two arrays.
 * @param[in]	x	Pointer the first vector of elements.
 * @param[in]	y	Pointer the second vector of elements.
 * @param[out]	dotProduct	Pointer the variable where the dot product values will be stored.
 * @param[in]	length	The length of the arrays specified by @a x and @a y.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a x, @a y or @a dotProduct argument is null.
 * @retval	#YepStatusMisalignedPointer	@a x, @a y or @a dotProduct argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE2, SSE3</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Haswell</td><td>AVX, FMA3</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX, FMA4</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_DotProduct_V64fV64f_S64f(const Yep64f *YEP_RESTRICT x, const Yep64f *YEP_RESTRICT y, Yep64f *YEP_RESTRICT dotProduct, YepSize length);
/**@}*/

/** @name	Sum of squares (squared L2 norm) */
/**@{*/
/**
 * @ingroup	yepCore
 * @brief	Computes the sum of squares of single precision (32-bit) floating-point elements in the input array.
 * @param[in]	v	Pointer the array of elements which will be squared (without write-back) and summed up.
 * @param[out]	sumSquares	Pointer a variable where the sum of squares will be stored.
 * @param[in]	length	The length of the array specified by @a v. The @a length is zero, the computed sum of squares will be 0.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a sumSquares argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a sumSquares argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE3</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX, FMA4</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_SumSquares_V32f_S32f(const Yep32f *YEP_RESTRICT v, Yep32f *YEP_RESTRICT sumSquares, YepSize length);
/**
 * @ingroup	yepCore
 * @brief	Computes the sum of squares of double precision (64-bit) floating-point elements in the input array.
 * @param[in]	v	Pointer the array of elements which will be squared (without write-back) and summed up.
 * @param[out]	sumSquares	Pointer a variable where the sum of squares will be stored.
 * @param[in]	length	The length of the array specified by @a v. The @a length is zero, the computed sum of squares will be 0.
 * @retval	#YepStatusOk	The computation finished successfully.
 * @retval	#YepStatusNullPointer	@a v or @a sumSquares argument is null.
 * @retval	#YepStatusMisalignedPointer	@a v or @a sumSquares argument is not naturally aligned.
 * @par	Optimized implementations
 * 		<table>
 * 			<tr><th>Architecture</th><th>Target microarchitecture</th><th>Required instruction extensions</th></tr>
 * 			<tr><td>x86-64</td><td>Intel Nehalem</td><td>SSE, SSE2, SSE3</td></tr>
 * 			<tr><td>x86-64</td><td>Intel Sandy Bridge</td><td>AVX</td></tr>
 * 			<tr><td>x86-64</td><td>AMD Bulldozer</td><td>AVX, FMA4</td></tr>
 * 		</table>
 */
YEP_PUBLIC_SYMBOL enum YepStatus YEPABI yepCore_SumSquares_V64f_S64f(const Yep64f *YEP_RESTRICT v, Yep64f *YEP_RESTRICT sumSquares, YepSize length);
/**@}*/

#ifdef __cplusplus
	} // extern "C"
#endif